"""
__version__.py
~~~~~~~~~~~~~~

Information about the current version of the py-package-template package.
"""

__title__ = "abst"
__description__ = (
    "CLI Command making OCI Bastion and kubernetes usage simple and fast"
)

__version__ = "2.3.48"
__author__ = "Jiri Otoupal"
__author_email__ = "jiri-otoupal@ips-database.eu"
__license__ = "MIT"
__url__ = "https://github.com/jiri-otoupal/abst"
__pypi_repo__ = "https://pypi.org/project/abst/"

__version_name__ = "SSHed Penguin"
__change_log__ = """
* 'abst ssh' will show screen with available instances for ssh from all abst processes\n
* moved old ssh to pod from 'abst ssh' to 'abst pod ssh'
"""
